@@ -29,7 +29,7 @@ def build_doc_for_new_epydoc():
     if OPTIONS:
         options = OPTIONS
     else:
-        options = '-v --html -o docs/api --graph=all %s' % packages
+        options = '-v --html -o docs/api --graph=classtree %s' % packages
     args = options.split(' ')
     epydocify(args)
 
