@@ -1,11 +1,11 @@
 # -*- coding: utf-8 -*-
 # Elisa - Home multimedia server
-# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
+# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
 # All rights reserved.
 #
 # This file is available under one of two license agreements.
 #
-# This file is licensed under the GPL version 2.
+# This file is licensed under the GPL version 3.
 # See "LICENSE.GPL" in the root of this distribution including a special
 # exception to use Elisa with Fluendo's plugins.
 #
@@ -40,6 +40,7 @@ class UriNotMonitorable(Exception):
 # MODIFIED: guess what, a directory or file was updated
 NotifyEvent = enum.Enum('ADDED', 'REMOVED', 'MODIFIED')
 
+media_types = ('directory', 'audio', 'video', 'image')
 
 class MediaProvider(Component):
     """
@@ -57,6 +58,12 @@ class MediaProvider(Component):
     TODO:
 
       - blocking_{copy,move,delete} ?
+
+    @cvar scannable_uri_schemes: DOCME
+    @type scannable_uri_schemes: list
+    @cvar supported_uri_schemes: DOCME
+    @type supported_uri_schemes: dict mapping string to int
+
     """
 
     def scannable_uri_schemes__get(self):
@@ -69,6 +76,9 @@ class MediaProvider(Component):
 
         By default the return value of this method corresponds to the
         keys of L{supported_uri_schemes__get} return value.
+
+        # FIXME: this should be documented in the class docstring as a class
+        # variable
         """
         return self.supported_uri_schemes.keys()
 
@@ -92,11 +102,12 @@ class MediaProvider(Component):
         another component should be used instead of it to access this
         scheme.
 
-        @rtype: dict mapping URI schemes (strings) to priorities
-                (positive integers)
-                example: { 'file': 0, 'smb': 10 }
-        """
+        example: { 'file': 0, 'smb': 10 }
 
+        # FIXME: this should be documented in the class docstring as a class
+        # variable
+        """
+        return {}
 
     def get_real_uri(self, uri):
         """
@@ -120,40 +131,41 @@ class MediaProvider(Component):
         @type uri:  L{elisa.core.media_uri.MediaUri}
         @rtype: L{twisted.internet.defer.Deferred}
         """
-        return defer.succeed(self.blocking_get_media_type(uri))
+        return defer.succeed(self._blocking_get_media_type(uri))
 
-    def blocking_get_media_type(self, uri):
+    def _blocking_get_media_type(self, uri):
         """
         Try to guess the maximum information from the media located
         at given uri by looking at eventual file extension. Will
         return something like::
 
-          {'file_type': string (values: 'directory', 'image', 'audio', 'video'),
+          {'file_type': string (values: one of media_provider.media_types)
            'mime_type': string (example: 'audio/mpeg' for .mp3 uris. can be
                                empty string if unguessable)
            }
 
-        This function is called when another component needs to know what kind
-        of media it is dealing with.
+        'file_type' and 'mime_type' can be empty strings if it failed
+        recognizing them.
 
         @param uri: the URI to analyze
         @type uri:  L{elisa.core.media_uri.MediaUri}
         @rtype:     dict
         """
+        raise NotImplementedError
     
     def is_directory(self, uri):
         """
-        Same as L{blocking_is_directory} but without blocking (in
+        Same as L{_blocking_is_directory} but without blocking (in
         theory). This method by default triggers a succeeded callback
-        on a Twisted deferred, using L{blocking_is_directory} result.
+        on a Twisted deferred, using L{_blocking_is_directory} result.
         
         @param uri: the URI to analyze
         @type uri:  L{elisa.core.media_uri.MediaUri}
         @rtype:     L{twisted.internet.defer.Deferred}
         """
-        return defer.succeed(self.blocking_is_directory(uri))
+        return defer.succeed(self._blocking_is_directory(uri))
     
-    def blocking_is_directory(self, uri):
+    def _blocking_is_directory(self, uri):
         """
         return True if a directory
 
@@ -161,12 +173,13 @@ class MediaProvider(Component):
         @type uri:  L{elisa.core.media_uri.MediaUri}
         @rtype:     bool
         """
+        raise NotImplementedError
 
     def has_children_with_types(self, uri, media_types):
         """
-        Same as L{blocking_has_children_with_types} but without blocking (in
+        Same as L{_blocking_has_children_with_types} but without blocking (in
         theory). This method by default triggers a succeeded callback
-        on a Twisted deferred, using L{blocking_has_children_with_types} result.
+        on a Twisted deferred, using L{_blocking_has_children_with_types} result.
         
         @param uri:         the URI to scan
         @type uri:          L{elisa.core.media_uri.MediaUri}
@@ -174,14 +187,14 @@ class MediaProvider(Component):
         @type media_types:  list of strings
         @rtype:             L{twisted.internet.defer.Deferred}
         """
-        return defer.succeed(self.blocking_has_children_with_types(uri,
+        return defer.succeed(self._blocking_has_children_with_types(uri,
                                                                    media_types))
     
-    def blocking_has_children_with_types(self, uri, media_types):
+    def _blocking_has_children_with_types(self, uri, media_types):
         """
         Detect whether the given uri has children for given media
-        types which can be of 'directory', 'audio', 'video',
-        'picture'. Implies the URI is a directory as well.
+        types which can be one of media_provider.media_types.
+        Implies the URI is a directory as well.
 
         @param uri:         the URI to scan
         @type uri:          L{elisa.core.media_uri.MediaUri}
@@ -189,12 +202,13 @@ class MediaProvider(Component):
         @type media_types:  list of strings
         @rtype:             bool
         """
+        raise NotImplementedError
 
     def get_direct_children(self, uri, children_with_info):
         """
-        Same as L{blocking_get_direct_children} but without blocking (in
+        Same as L{_blocking_get_direct_children} but without blocking (in
         theory). This method by default triggers a succeeded callback
-        on a Twisted deferred, using L{blocking_get_direct_children} result.
+        on a Twisted deferred, using L{_blocking_get_direct_children} result.
         
         @param uri:                the URI to analyze
         @type uri:                 L{elisa.core.media_uri.MediaUri}
@@ -202,10 +216,10 @@ class MediaProvider(Component):
         @type children_with_info:  list
         @rtype:                    L{twisted.internet.defer.Deferred}
         """
-        return defer.succeed(self.blocking_get_direct_children(uri,
+        return defer.succeed(self._blocking_get_direct_children(uri,
                                                                children_with_info))
     
-    def blocking_get_direct_children(self, uri, children_with_info):
+    def _blocking_get_direct_children(self, uri, children_with_info):
         """
         Scan the data located at given uri and return informations
         about its children.  Fills children_with_info.
@@ -236,9 +250,9 @@ class MediaProvider(Component):
 
     def open(self, uri, mode=None):
         """
-        Same as L{blocking_open} but without blocking (in
+        Same as L{_blocking_open} but without blocking (in
         theory). This method by default triggers a succeeded callback
-        on a Twisted deferred, using L{blocking_open} result.
+        on a Twisted deferred, using L{_blocking_open} result.
 
         @note: It's not allowed to open directories, it's up to the
         developer to check that the URI to open doesn't represent a
@@ -250,9 +264,9 @@ class MediaProvider(Component):
         @type mode:     string or None
         @rtype:         L{twisted.internet.defer.Deferred}
         """
-        return defer.succeed(self.blocking_open(uri, mode=mode))
+        return defer.succeed(self._blocking_open(uri, mode=mode))
     
-    def blocking_open(self, uri, mode=None):
+    def _blocking_open(self, uri, mode=None):
         """
         Open an uri and return MediaFile.
 
@@ -267,20 +281,21 @@ class MediaProvider(Component):
         @type mode:     string or None
         @rtype:         L{elisa.core.media_file.MediaFile}
         """
+        raise NotImplementedError
 
     def close(self, media_file):
         """
-        Same as L{blocking_close} but without blocking (in
+        Same as L{_blocking_close} but without blocking (in
         theory). This method by default triggers a succeeded callback
-        on a Twisted deferred, using L{blocking_close} result.
+        on a Twisted deferred, using L{_blocking_close} result.
 
         @param media_file: the file to close
         @type media_file:  L{elisa.core.media_file.MediaFile}
         @rtype:            L{twisted.internet.defer.Deferred}
         """
-        return defer.succeed(self.blocking_close(media_file))
+        return defer.succeed(self._blocking_close(media_file))
         
-    def blocking_close(self, media_file):
+    def _blocking_close(self, media_file):
         """
         Close a MediaFile
 
@@ -292,9 +307,9 @@ class MediaProvider(Component):
 
     def seek(self, media_file, offset, whence=0):
         """
-        Same as L{blocking_seek} but without blocking (in
+        Same as L{_blocking_seek} but without blocking (in
         theory). This method by default triggers a succeeded callback
-        on a Twisted deferred, using L{blocking_seek} result.
+        on a Twisted deferred, using L{_blocking_seek} result.
 
         @param media_file: the file to seek in
         @type media_file:  L{elisa.core.media_file.MediaFile}
@@ -304,10 +319,10 @@ class MediaProvider(Component):
         @type whence:      int (default=0)
         @rtype:            L{twisted.internet.defer.Deferred}
         """
-        return defer.succeed(self.blocking_seek(media_file, offset,
+        return defer.succeed(self._blocking_seek(media_file, offset,
                                                 whence=whence))
 
-    def blocking_seek(self, media_file, offset, whence=0):
+    def _blocking_seek(self, media_file, offset, whence=0):
         """
         Seek data in a MediaFile
 
@@ -323,9 +338,9 @@ class MediaProvider(Component):
 
     def read(self, media_file, size=-1):
         """
-        Same as L{blocking_seek} but without blocking (in
+        Same as L{_blocking_seek} but without blocking (in
         theory). This method by default triggers a succeeded callback
-        on a Twisted deferred, using L{blocking_seek} result.
+        on a Twisted deferred, using L{_blocking_seek} result.
 
         @param media_file: the file to read data from
         @type media_file:  L{elisa.core.media_file.MediaFile}
@@ -333,9 +348,9 @@ class MediaProvider(Component):
         @type size:        int (default: -1 == all)
         @rtype:            L{twisted.internet.defer.Deferred}
         """
-        return defer.succeed(self.blocking_read(media_file, size=size))
+        return defer.succeed(self._blocking_read(media_file, size=size))
 
-    def blocking_read(self, media_file, size=-1):
+    def _blocking_read(self, media_file, size=-1):
         """
         Read data from a MediaFile
 
@@ -355,9 +370,9 @@ class MediaProvider(Component):
 
     def next_location(self, uri, root=None):
         """
-        Same as L{blocking_next_location} but without blocking (in
+        Same as L{_blocking_next_location} but without blocking (in
         theory). This method by default triggers a succeeded callback
-        on a Twisted deferred, using L{blocking_next_location} result.
+        on a Twisted deferred, using L{_blocking_next_location} result.
         
         @param uri:  the URI representing the file or directory from
                      where to move on
@@ -366,9 +381,9 @@ class MediaProvider(Component):
         @type root:  L{elisa.core.media_uri.MediaUri}
         @rtype:      L{twisted.internet.defer.Deferred}
         """
-        return defer.succeed(self.blocking_next_location(uri, root=root))
+        return defer.succeed(self._blocking_next_location(uri, root=root))
     
-    def blocking_next_location(self, uri, root=None):
+    def _blocking_next_location(self, uri, root=None):
         """
         Return the uri just next to given uri.
 
@@ -379,21 +394,22 @@ class MediaProvider(Component):
         @type root:  L{elisa.core.media_uri.MediaUri}
         @rtype:      L{elisa.core.media_uri.MediaUri}
         """
+        raise NotImplementedError
 
     def previous_location(self, uri):
         """
-        Same as L{blocking_previous_location} but without blocking (in
+        Same as L{_blocking_previous_location} but without blocking (in
         theory). This method by default triggers a succeeded callback
-        on a Twisted deferred, using L{blocking_previous_location} result.
+        on a Twisted deferred, using L{_blocking_previous_location} result.
         
         @param uri: the URI representing the file or directory prior
                     to uri
         @type uri:  L{elisa.core.media_uri.MediaUri}
         @rtype:     L{twisted.internet.defer.Deferred}
         """
-        return defer.succeed(self.blocking_next_location(uri, root=root))
+        return defer.succeed(self._blocking_next_location(uri, root=root))
 
-    def blocking_previous_location(self, uri):
+    def _blocking_previous_location(self, uri):
         """
         Return the uri found before given uri
 
@@ -401,6 +417,7 @@ class MediaProvider(Component):
         @type uri:  L{elisa.core.media_uri.MediaUri}
         @rtype:     L{elisa.core.media_uri.MediaUri}
         """
+        raise NotImplementedError
 
     def monitor_uri(self, uri, callback):
         """
@@ -410,9 +427,6 @@ class MediaProvider(Component):
 
         @param uri:      URI representing the file or directory to monitor
         @type uri:       L{elisa.core.media_uri.MediaUri}
-        @param extra_args: extra positional arguments to pass to the callback
-        @type extra_args: tuple
-
         @param callback: a callable taking the event that occured and the uri
                          of the file on which the event applies to
                          prototype: callable(uri, metadata, event)
@@ -422,6 +436,7 @@ class MediaProvider(Component):
 
         @raise UriNotMonitorable : if the uri cannot be monitored
         """
+        raise NotImplementedError
 
     def unmonitor_uri(self, uri):
         """
@@ -430,6 +445,7 @@ class MediaProvider(Component):
         @param uri: the URI representing the file or directory to monitor
         @type uri:  L{elisa.core.media_uri.MediaUri}
         """
+        raise NotImplementedError
 
     def uri_is_monitorable(self, uri):
         """
@@ -440,6 +456,7 @@ class MediaProvider(Component):
         @type uri:  L{elisa.core.media_uri.MediaUri}
         @rtype:     bool
         """
+        return False
 
     def uri_is_monitored(self, uri):
         """
@@ -451,6 +468,7 @@ class MediaProvider(Component):
         @type uri:  L{elisa.core.media_uri.MediaUri}
         @rtype:     bool
         """
+        raise NotImplementedError
 
     def copy(self, orig_uri, dest_uri, recursive=False):
         """
@@ -469,6 +487,7 @@ class MediaProvider(Component):
         @type recursive:  bool
         @rtype:           L{twisted.internet.defer.Deferred}
         """
+        raise NotImplementedError
 
     def move(self, orig_uri, dest_uri):
         """
@@ -485,6 +504,7 @@ class MediaProvider(Component):
         @type dest_uri:  L{elisa.core.media_uri.MediaUri}
         @rtype:          L{twisted.internet.defer.Deferred}
         """
+        raise NotImplementedError
 
     def delete(self, uri, recursive=False):
         """
@@ -501,3 +521,4 @@ class MediaProvider(Component):
         @type recursive:  bool
         @rtype:           L{twisted.internet.defer.Deferred}
         """
+        raise NotImplementedError
