@@ -1,11 +1,11 @@
 # -*- coding: utf-8 -*-
 # Elisa - Home multimedia server
-# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
+# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
 # All rights reserved.
 #
 # This file is available under one of two license agreements.
 #
-# This file is licensed under the GPL version 2.
+# This file is licensed under the GPL version 3.
 # See "LICENSE.GPL" in the root of this distribution including a special
 # exception to use Elisa with Fluendo's plugins.
 #
@@ -172,13 +172,16 @@ class MediaDB(log.Loggable):
 
         self._backend.sql_execute("insert into core_meta(version) values(?)",
                                   CURRENT_DB_VERSION)
+        self._backend.save_changes()
         self._backend.reconnect()
 
-    def get_files_count_for_source(self, source):
+    def get_files_count_for_source_uri(self, source_uri):
         """
         DOCME
         """
-        request = "select count(*) as c from core_media where typ='file' and source_id=%s" % source.id
+        request = "select count(*) as c from core_media media, " \
+                "core_source source where source.uri = '%s' and " \
+                "media.source_id = source.id" % source_uri
         count = self._backend.sql_execute(request)[0].c
         return count
 
@@ -316,25 +319,24 @@ class MediaDB(log.Loggable):
 
             extras_spc, extras_val, extras_marks = self._kw_extract(extra)
             if extras_spc:
-                req = "insert into core_media(uri, updated, date_added, short_name,typ, %s) values (?,1, current_timestamp, ?,?,%s)"
+                req = "insert or replace into core_media(uri, updated, date_added, short_name,typ, %s) values (?,1, current_timestamp, ?,?,%s)"
                 req = req % (extras_spc, extras_marks)
             else:
-                req = "insert into core_media(uri, updated, date_added, short_name,typ) values (?, 1, current_timestamp, ?,?)"
+                req = "insert or replace into core_media(uri, updated, date_added, short_name,typ) values (?, 1, current_timestamp, ?,?)"
 
             id = self._backend.insert(req, uri, short_name, content_type,
                                       *extras_val)
-            self._backend.save_changes()
 
             # add new row in specialized table
             format = extra.get('format')
             if format:
                 spec, values, marks = self._kw_extract(metadata)
                 if spec and values:
-                    req = "insert into core_%s_media(media_id,%s) values (?,%s)"
+                    req = "insert or replace into core_%s_media(media_id,%s) values (?,%s)"
                     req = req % (format, spec, marks)
                     args = (id,) + tuple(values)
                 else:
-                    req = "insert into core_%s_media(media_id) values (?)" % format
+                    req = "insert or replace into core_%s_media(media_id) values (?)" % format
                     args = (id,)
                 self._backend.insert(req, *args)
             return True
@@ -641,10 +643,10 @@ class MediaDB(log.Loggable):
         else:   
             
             if select_attribute == 'files':
-                select_clause = 'track, song, uri'
+                select_clause = 'track, song, uri, cover_uri'
             else:
                 select_clause = self.available_meta[select_attribute]
-                
+
             query, args = self._build_request(select_clause, path_values, start, item_count)
             rows = self._backend.sql_execute(query, *args)
  
@@ -665,7 +667,14 @@ class MediaDB(log.Loggable):
                 if select_attribute == 'albums':
                     metadata['album'] = row[0]
                     metadata['artist'] = self.get_artist_from_album(row[0])
-
+                elif select_attribute == 'files':
+                    default_image = row[3]
+                    if default_image == u'':
+                        default_image = None
+                    else:
+                        default_image = MediaUri(default_image)
+                    metadata['default_image'] = default_image
+                
                 new_uri = MediaUri(unicode(child_uri))
                 if child_label:
                     new_uri.label = child_label
