@@ -1,11 +1,11 @@
 # -*- coding: utf-8 -*-
 # Elisa - Home multimedia server
-# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
+# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
 # All rights reserved.
 #
 # This file is available under one of two license agreements.
 #
-# This file is licensed under the GPL version 2.
+# This file is licensed under the GPL version 3.
 # See "LICENSE.GPL" in the root of this distribution including a special
 # exception to use Elisa with Fluendo's plugins.
 #
@@ -117,17 +117,12 @@ MediaManager & MediaProvider notes and ideas
 """
 
 import threading, os
-from twisted.internet import reactor
+from twisted.internet import reactor, defer
 
 from elisa.core import common, manager
 from elisa.core.utils import classinit
 from elisa.base_components import media_provider
-from elisa.core import media_scanner
 from elisa.core import media_uri
-from elisa.core import db_backend
-from elisa.core.media_db import MediaDB
-
-DB_CONNECTIONS = {}
 
 class MediaProviderNotFound(Exception):
 
@@ -165,59 +160,8 @@ class MediaManager(manager.Manager):
         @type metadata_manager: L{elisa.core.metadata_manager.MetadataManager}
         """
         manager.Manager.__init__(self)
-        self._media_scanner = media_scanner.MediaScanner(self, metadata_manager)
-        self.source_updated = self._media_scanner.source_updated
-        self._scanner_enabled = self._media_scanner.enabled
-
-        self._db_ok = True
-        self._db_lock = threading.Lock()
+        self._metadata_manager = metadata_manager
         self._providers_by_scheme = {}
-        self._delayed_start = None
-        
-    def media_db__get(self):
-        """ MediaDB instance accessor.
-        """
-        db = None
-
-        global DB_CONNECTIONS
-        try:
-            self._db_lock.acquire()
-            current_thread = threading.currentThread()
-            thread_id = id(current_thread)
-            backend = DB_CONNECTIONS.get(thread_id)
-            first_load = False
-            if not backend:
-                first_load = True
-                opts = self._get_db_options()
-                try:
-                    backend = db_backend.DBBackend(**opts)
-                except db_backend.DBBackendError, error:
-                    self.warning(error)
-                    self._db_ok = False
-                else:
-                    DB_CONNECTIONS[thread_id] = backend
-            if backend:
-                db = MediaDB(backend, first_load)
-        finally:
-            self._db_lock.release()
-
-        return db
-
-    def _get_db_options(self):
-        config = self._media_scanner._config
-        config_dir = common.application.config.get_config_dir()
-        
-        keys = ['db_backend', 'database', 'username', 'password', 'hostname']
-        opts = dict([(k,v) for k,v in config.iteritems() if k in keys])
-
-        # SQLite db should be in same directory as the elisa config file.
-        if opts['db_backend'] == 'sqlite':
-            opts['database'] = os.path.join(config_dir, opts['database'])
-            
-        return opts
-
-    def enabled__get(self):
-        return self._db_ok or self._scanner_enabled
 
     def start(self, seconds=0, resume_scan=False):
         """
@@ -228,33 +172,17 @@ class MediaManager(manager.Manager):
         @type seconds:        int
         @keyword resume_scan: should the media_scanner resume interrupted
                               scan at startup?
-        @type resume:         bool
+        @type resume_scan:    bool
         """
         
-        def _start():
-            if self._scanner_enabled:
-                self.info('Starting')
-                self._media_scanner.start(auto_resume=resume_scan)
-
-        self._media_scanner.delayed_start = seconds
-        self._delayed_start = reactor.callLater(seconds, _start)
-
     def stop(self):
         """
         Stop the media_scanner if it's running and clean all
         registered media_providers.
         """
         self.info('Stopping')
-
-        if self._delayed_start:
-            try:
-                self._delayed_start.cancel()
-            except:
-                pass
-
-        if self._scanner_enabled:
-            self._media_scanner.stop()
-            manager.Manager.stop(self)
+        
+        return manager.Manager.stop(self)
 
     def register_component(self, component):
         try:
@@ -281,99 +209,9 @@ class MediaManager(manager.Manager):
             pass
             #FIXME: needs to be raised?
 
-    def get_metadata(self, metadata):
+    def get_metadata(self, metadata, low_priority=False):
         """just a proxy to the metadata_manager.get_metadata"""
-        return self._media_scanner.get_metadata(metadata)
-
-    def add_source(self, source_uri, media_types, callback=None):
-        """
-        Add a new source in DB. If it's already there, just mark it as
-        available again (and start monitoring it).
-
-        @param source_uri: The location of the source
-        @type source_uri:  L{elisa.core.media_uri.MediaUri}
-        @param callback:   Function called by the MediaScanner when it needs to
-                           process files and directories
-        @type callback:    callable(media_uri, event_type, file_type, media_type)
-        """
-        source = None
-        if self.enabled:
-            source = self.get_source_for_uri(source_uri)
-            if not source:
-                short_name = source_uri.label
-                source = self.media_db.add_source(source_uri, short_name)
-
-            source = self._media_scanner.add_source(source, media_types,
-                                                    callback=callback)
-        return source
-    
-    def remove_source(self, source_uri):
-        """
-        Mark a source as unavailable in the db and stop monitoring it
-
-        @param source_uri: The location of the source
-        @type source_uri:  L{elisa.core.media_uri.MediaUri}
-        """
-        if self.enabled:
-            source = self.get_source_for_uri(source_uri)
-            if source:
-                # mark the source as unavailable in the db
-                self.media_db.hide_source(source)
-
-                self._media_scanner.remove_source(source_uri)
-
-    def update_source(self, source_uri):
-        """
-        Schedule a new scan of the source located at given uri. The
-        scan may not start just after calling this method.
-
-        @param source_uri: The location of the source
-        @type source_uri:  L{elisa.core.media_uri.MediaUri}
-        """
-        if self.enabled:
-            return self._media_scanner.update_source(source_uri)
-
-    def get_source_for_uri(self, uri):
-        """
-        Fetch the source located at given uri, by retrieving its
-        information in the media database.
-
-        @param uri: The uri of the source to look for
-        @type uri:  L{elisa.core.media_uri.MediaUri}
-        @rtype:     L{elisa.extern.db_row.DBRow}
-        """
-        if self.enabled:
-            return self.media_db.get_source_for_uri(uri)
-
-    def handle_notify_event(self, uri, event, media_types):
-        if self.enabled:
-            parent_uri = media_uri.MediaUri(uri.parent[:-1])
-            media = self.media_db.get_media_information(parent_uri)
-            #source = None
-            if media:
-                source = self.media_db.get_source_with_id(media.source_id)
-            else:
-                source = self.media_db.get_source_for_uri(parent_uri)
-
-            if source:
-                if event == media_provider.NotifyEvent.ADDED:
-                    self._media_scanner.add(source, uri)
-                elif event == media_provider.NotifyEvent.REMOVED:
-                    self._media_scanner.remove(source, uri)
-
-    def get_media_information(self, uri, extended=True, media_type=None):
-        """
-        Fetch the media located at given uri, by retrieving its
-        information in the media database.
-
-        @param uri: The uri of the media to look for
-        @type uri:  L{elisa.core.media_uri.MediaUri}
-        @rtype:     L{elisa.extern.db_row.DBRow}
-        """
-        if self.enabled:
-            return self.media_db.get_media_information(uri,
-                                                       media_type=media_type,
-                                                       extended=extended)
+        return self._metadata_manager.get_metadata(metadata, low_priority=False)
 
     def _get_media_provider(self, uri):
         """
@@ -411,13 +249,35 @@ class MediaManager(manager.Manager):
         scannable = provider and uri.scheme in provider.scannable_uri_schemes
         return scannable
 
+    def _guess_source_for_uri(self, uri):
+        source = None
+        source_uri = uri
+
+        while True:
+            u_uri = unicode(source_uri)
+            if u_uri[-1] == '/':
+                u_uri = u_uri[:-1]
+
+            source = self.get_source_for_uri(u_uri)
+            if source:
+                break
+            
+            source = None
+            parent = source_uri.parent
+            if parent == source_uri:
+                break
+
+            source_uri = parent
+
+        return source
+
     def get_media_type(self, uri):
         """
         Try to guess the maximum information from the media located
         at given uri by looking at eventual file extension. Will
         return something like::
 
-          {'file_type': string (values: 'directory', 'image', 'audio', 'video'),
+          {'file_type': string (values: one of media_provider.media_types,
            'mime_type': string (example: 'audio/mpeg' for .mp3 uris. can be
                                empty string if unguessable)
            }
@@ -427,10 +287,11 @@ class MediaManager(manager.Manager):
         @type uri:  L{elisa.core.media_uri.MediaUri}
         @rtype:     L{twisted.internet.defer.Deferred}
         """
+
         return self._proxy('get_media_type', {}, uri)
-    
+
     def blocking_get_media_type(self, uri):
-        return self._proxy('blocking_get_media_type', {}, uri)
+        return self._proxy('_blocking_get_media_type', {}, uri)
 
     def is_directory(self, uri):
         """
@@ -443,13 +304,13 @@ class MediaManager(manager.Manager):
         return self._proxy('is_directory', False, uri)
 
     def blocking_is_directory(self, uri):
-        return self._proxy('blocking_is_directory', False, uri)
+        return self._proxy('_blocking_is_directory', False, uri)
 
     def has_children_with_types(self, uri, media_types):
         """
         Detect whether the given uri has children for given media
-        types which can be of 'directory', 'audio', 'video',
-        'picture'. Implies the URI is a directory as well.
+        types which can be one of media_provider.media_types.
+        Implies the URI is a directory as well.
 
         @param uri:         the URI to scan
         @type uri:          L{elisa.core.media_uri.MediaUri}
@@ -460,7 +321,7 @@ class MediaManager(manager.Manager):
         return self._proxy('has_children_with_types', False, uri, media_types)
 
     def blocking_has_children_with_types(self, uri, media_types):
-        return self._proxy('blocking_has_children_with_types',
+        return self._proxy('_blocking_has_children_with_types',
                            False, uri, media_types)
 
     def get_direct_children(self, uri, children_with_info):
@@ -486,7 +347,7 @@ class MediaManager(manager.Manager):
         return self._proxy('get_direct_children', None, uri, children_with_info)
 
     def blocking_get_direct_children(self, children_with_info):
-        return self._proxy('blocking_get_direct_children', None, uri,
+        return self._proxy('_blocking_get_direct_children', None, uri,
                            children_with_info)
 
     def open(self, uri, mode='r'):
@@ -505,7 +366,7 @@ class MediaManager(manager.Manager):
 
 
     def blocking_open(self, uri, mode='r'):
-        return self._proxy('blocking_open', None, uri, mode=mode)
+        return self._proxy('_blocking_open', None, uri, mode=mode)
 
     def next_location(self, uri, root=None):
         """
@@ -536,7 +397,7 @@ class MediaManager(manager.Manager):
 
     def blocking_next_location(self, uri, root=None):
         location = None
-        method_name = 'blocking_next_location'
+        method_name = '_blocking_next_location'
         if root:
             provider = self._get_media_provider(root)
         else:
@@ -558,7 +419,7 @@ class MediaManager(manager.Manager):
         return self._proxy('previous_location', None, uri)
 
     def blocking_previous_location(self, uri):
-        return self._proxy('blocking_previous_location', None, uri)
+        return self._proxy('_blocking_previous_location', None, uri)
 
     def monitor_uri(self, uri, callback, *extra_args):
         """
