@@ -1,11 +1,11 @@
 # -*- coding: utf-8 -*-
 # Elisa - Home multimedia server
-# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
+# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
 # All rights reserved.
 #
 # This file is available under one of two license agreements.
 #
-# This file is licensed under the GPL version 2.
+# This file is licensed under the GPL version 3.
 # See "LICENSE.GPL" in the root of this distribution including a special
 # exception to use Elisa with Fluendo's plugins.
 #
@@ -21,7 +21,7 @@ URI parsing support
 __maintainer__ = 'Philippe Normand <philippe@fluendo.com>'
 
 
-import os, re, urllib
+import os, re, urllib, platform, copy
 from elisa.core.utils import classinit
 
 # for i18n support
@@ -94,24 +94,42 @@ def unquote(data):
         if isinstance(data, unicode):
             # urllib.quote doesn't support unicode objects very well
             data = data.encode('utf-8')
-        unquoted = unicode(urllib.unquote(data.decode('utf-8')))
+        unquoted = urllib.unquote(data).decode('utf-8')
     elif isinstance(data, dict):
         unquoted = {}
         for k, v in data.iteritems():
             if isinstance(v, unicode):
                 # urllib.quote doesn't support unicode objects very well
                 v = v.encode('utf-8')
-            unquoted[k] = unicode(urllib.unquote(v.decode('utf-8')))
+            unquoted[k] = urllib.unquote(v).decode('utf-8')
     elif isinstance(data, list):
         unquoted = []
         for entry in data:
             if isinstance(entry, unicode):
                 entry = entry.encode('utf-8')
-            unquoted.append(unicode(urllib.unquote(entry.decode('utf-8'))))
+            unquoted.append(urllib.unquote(entry).decode('utf-8'))
     else:
         unquoted = data
     return unquoted
 
+def _unicode(data):
+    """
+    Convert data to unicode using utf-8 encoding instead of ascii.
+
+    @param data: data to convert
+    @type data:  string, unicode or other object
+    @rtype:      unicode
+    """
+    if isinstance(data, unicode):
+        result = data
+    elif isinstance(data, str):
+        result = unicode(data, 'utf-8')
+    else:
+        result = unicode(str(data), 'utf-8')
+    return result
+        
+        
+
 class MediaUri:
     """ Media URI management
 
@@ -184,42 +202,52 @@ class MediaUri:
 
         if isinstance(data, MediaUri):
             # copycat, avoid another parse
+            if data.path.find('\\') != -1:
+                path = data.path.replace('\\', '/')
+            else:
+                path = data.path
+                
             uri_parts = {'scheme': data.scheme, 'user': data.user,
                          'password': data.password, 'host': data.host,
-                         'port': data.port, 'path': data.path,
-                         'fragment': data.fragment, 'params': data._params}
+                         'port': data.port, 'path': path,
+                         'fragment': data.fragment,
+                         'params': copy.copy(data._params)}
             self._label = data._label
         elif isinstance(data, dict):
             # FIXME: scheme, host and path cannot be empty according to the docstring
+            if data.has_key('path') and data['path'].find('\\') != -1:
+                data['path'] = data['path'].replace('\\', '/')
             uri_parts = data
         elif isinstance(data, str):
-            location = unicode(data)
+            
+            if data.find('\\') != -1:
+                data = data.replace('\\', '/')            
+            location = unicode(data, 'utf-8')
         elif isinstance(data, unicode):
             location = data
 
         if uri_parts:
             # FIXME: scheme cannot be empty according to the docstring
-            self.scheme = unicode(uri_parts['scheme'])
-            self.user = unicode(uri_parts.get('user', ''))
-            self.password = unicode(uri_parts.get('password', ''))
+            self.scheme = _unicode(uri_parts['scheme'])
+            self.user = _unicode(uri_parts.get('user', ''))
+            self.password = _unicode(uri_parts.get('password', ''))
             # FIXME: host cannot be empty according to the docstring
             #self.host = unicode(uri_parts['host'])
-            self.host = unicode(uri_parts.get('host', ''))
+            self.host = _unicode(uri_parts.get('host', ''))
             # port is an integer, according to the docstring
             self.port = uri_parts.get('port', 0)
 
-            self.fragment = unicode(uri_parts.get('fragment', ''))
+            self.fragment = _unicode(uri_parts.get('fragment', ''))
             # FIXME: convert params to unicode
             self._params = uri_parts.get('params', {})
 
             path = uri_parts['path']
-
             # path should always start with / if it's not null
             # FIXME: why: Sorry, but this unusefull and not at is used in other cases. For e.g. the cdda-uri of gstreamer is: cdda://track-num. This is not working correctly...
             if path and not path.startswith('/'):
                 path = "/%s" % path
 
-            self.path = unicode(path)
+            self._path = _unicode(path)
         elif location:
             self._parse(location)
         else:
@@ -234,7 +262,17 @@ class MediaUri:
                                 "(\?([^#]*))?(#(.*))?", re.UNICODE)
         return URI_RE
 
+    def path__get(self):
+        if platform.system() == 'Windows' and len(self._path) > 0 and self._path[0] == '/' and self.scheme == 'file':
+            return self._path[1:]
+        return self._path
+    
+    def path__set(self, value):
+        self._path = value
+    
     def _parse(self, location):
+        
+        system_name = platform.system()
         match = self._uri_re().search(location)
         if match:
             port = 0
@@ -255,10 +293,14 @@ class MediaUri:
             else:
                 self.password = u''
 
+
             host_port = host.split(':')
             if len(host_port) == 2:
                 host, port = host_port
-                port = int(port)
+                try:
+                    port = int(port)
+                except:
+                    port = -1
 
             if host:
                 self.host = host
@@ -280,6 +322,9 @@ class MediaUri:
                     if path.startswith('/'):
                         path = path[1:]
                     path = os.path.join(os.getcwd(), path)
+                    #For windows system
+                    if system_name == 'Windows':
+                        path = path.replace('\\', '/')
 
                 elif self.host:
                     # transfer host to beginning of path
@@ -290,7 +335,10 @@ class MediaUri:
                     # path should always start with /
                     path = "/%s" % path
 
-            self.path = unicode(path)
+                #remove double // in path
+                path = path.replace('//','/')
+                
+            self._path = _unicode(path)
 
             if params:
                 key_values = params.split('&')
@@ -304,7 +352,7 @@ class MediaUri:
                     key_values = params.split(',')
 
                 try:
-                    self._params = dict([map(unicode, i.split('='))
+                    self._params = dict([map(_unicode, i.split('='))
                                         for i in key_values])
                 except Exception, e:
                     msg = "URI parameters were not quoted in %s" % location
@@ -358,6 +406,22 @@ class MediaUri:
         """
         return unquote(self._params.get(name, default))
 
+    def get_params(self):
+        """
+        Get all params as a dict.
+        @rtype:         dict
+        @return:        of quoted key value pairs
+        """
+        return self._params
+
+    def get_params_string(self):
+        """
+        Get the params as a one-line string (excluding a leading '?').
+        @rtype:         unicode
+        @return:        key1=value1&key2=value2
+        """
+        return u'&'.join('='.join((k,v)) for k,v in self._params.iteritems())
+
     def del_param(self, name):
         """
         Delete the paramter with the name 'name' (and it's value), if it is
@@ -371,11 +435,11 @@ class MediaUri:
 
     def extension__get(self):
         # FIXME: is os.path.splitext doing any encoding conversion ?
-        dummy, ext = os.path.splitext(self.path)
+        dummy, ext = os.path.splitext(self._path)
         if ext:
             # strip '.' prefix
             ext = ext[1:].lower()
-        return unicode(ext)
+        return _unicode(ext)
 
     def filename__get(self):
         """ Return the filename of the Uri.
@@ -389,16 +453,16 @@ class MediaUri:
         """
         filename = ''
 
-        if self.path and self.path != '/':
-            idx = self.path.rfind('/')
+        if self._path and self._path != '/':
+            idx = self._path.rfind('/')
             if idx != -1:
-                filename = self.path[idx+1:]
+                filename = self._path[idx+1:]
 
-        return unicode(filename)
+        return _unicode(filename)
 
     def label__set(self, value):
         if isinstance(value, str):
-            self._label = value.decode('utf-8')#unicode(value)
+            self._label = _unicode(value)
         elif isinstance(value, unicode):
             self._label = value
         elif isinstance(value, Translatable):
@@ -410,10 +474,10 @@ class MediaUri:
         """
         label = self._label
         if not label:
-            path = self.path
+            path = self._path
 
             if path == '/':
-                label = unicode(path)
+                label = _unicode(path)
             elif path == '':
                 label = self.host
             else:
@@ -422,7 +486,7 @@ class MediaUri:
 
                 idx = path.rfind('/')
                 if idx != -1:
-                    label = unicode(unquote(path[idx+1:]))
+                    label = _unicode(unquote(path[idx+1:]))
 
         return label
 
@@ -432,13 +496,13 @@ class MediaUri:
 
         If the URI is like I{uri://path/to/foo} return I{uri://path/to/}
         """
-        p = self.path
+        p = self._path
         if p.endswith('/'):
             p = p[:-1]
         s = p.split('/')[:-1]
         p = '/'.join(s) + '/'
 
-        path = unicode(p)
+        path = _unicode(p)
 
         uri_parts = {'scheme': self.scheme,
                      'user': self.user,
@@ -455,12 +519,12 @@ class MediaUri:
         @type path:  string
         @rtype:      L{MediaUri}
         """
-        new_path = self.path
+        new_path = self._path
         if not new_path.endswith('/'):
             new_path += '/'
         new_path += path
 
-        new_path = unicode(new_path)
+        new_path = _unicode(new_path)
 
         uri_parts = {'scheme': self.scheme,
                      'user': self.user,
@@ -484,7 +548,7 @@ class MediaUri:
             if self.port:
                 ret += u':%s' % self.port
 
-        ret += self.path
+        ret += self._path
         if len(self._params):
             ret += u'?'
             if type(self._params) != dict:
@@ -560,9 +624,9 @@ class MediaUri:
     def __getslice__(self, i, j):
         return self._to_unicode()[i:j]
 
-if __name__ == '__main__':
-    import sys
-    for uri in sys.argv[1:]:
-        u = MediaUri(uri)
-        print u, u.label
-        assert str(u) == uri, "%r != %r" % (u, uri)
+#if __name__ == '__main__':
+#    import sys
+#    for uri in sys.argv[1:]:
+#        u = MediaUri(uri)
+#        print u, u.label
+#        assert str(u) == uri, "%r != %r" % (u, uri)
