@@ -1,11 +1,11 @@
 # -*- coding: utf-8 -*-
 # Elisa - Home multimedia server
-# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
+# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
 # All rights reserved.
 #
 # This file is available under one of two license agreements.
 #
-# This file is licensed under the GPL version 2.
+# This file is licensed under the GPL version 3.
 # See "LICENSE.GPL" in the root of this distribution including a special
 # exception to use Elisa with Fluendo's plugins.
 #
@@ -18,7 +18,8 @@
 __maintainer__ = 'Benjamin Kampmann <benjamin@fluendo.com>'
 
 
-from elisa.core.bus import bus, bus_message
+from elisa.core.bus import bus
+from elisa.base_components import message
 from elisa.core import common, log
 from elisa.extern import enum
 
@@ -26,19 +27,19 @@ from elisa.core.utils import classinit
 
 from elisa.core.player_engine_registry import NoEngineFound
 
-import pygst
-pygst.require('0.10')
+#import pygst
+#pygst.require('0.10')
 import gst
 
 from gobject import GError
 
-class PlayerPlaying(bus_message.Message):
+class PlayerPlaying(message.Message):
     """
     Sent over the message bus when the first frame or buffer of a media has
     been outputted.
     """
 
-class PlayerStopping(bus_message.Message):
+class PlayerStopping(message.Message):
     """
     Sent over the message bus when the playing is stopped.
 
@@ -53,13 +54,13 @@ class PlayerStopping(bus_message.Message):
         @param percent: the percent of the media, that was been watched yet
         @type percent: float
         """
-        bus_message.Message.__init__(self)
+        message.Message.__init__(self)
         self.percent = percent
 
     # FIXME: the reason why it was stopped could be useful (like EOS/EOF,
     #        Error, StreamBroke or Requested)
 
-class PlayerLoading(bus_message.Message):
+class PlayerLoading(message.Message):
     """
     Sent over the message bus when the player is asked to play. When the
     playback B{really} starts, that is when the first buffer reach the sink,
@@ -71,7 +72,7 @@ class PlayerLoading(bus_message.Message):
     Warning: this message is B{not} sent when the uri is set.
     """
 
-class PlayerBuffering(bus_message.Message):
+class PlayerBuffering(message.Message):
     """
     Sent if the player is loading before playing or if it is prebuffering during
     playing. This message might be sent very often and is mainly useful for
@@ -88,7 +89,7 @@ class PlayerBuffering(bus_message.Message):
     __classinit__ = classinit.build_properties
 
     def __init__(self,progress):
-        bus_message.Message.__init__(self)
+        message.Message.__init__(self)
         self._progress = 0
         self.progress = progress
 
@@ -103,12 +104,12 @@ class PlayerBuffering(bus_message.Message):
         else:
             self._progress = progress
 
-class PlayerPausing(bus_message.Message):
+class PlayerPausing(message.Message):
     """
     Sent over the message bus when the playing is paused.
     """
 
-class NewClock(bus_message.Message):
+class NewClock(message.Message):
     """
     This message is triggered everytime the engine gets a new Clock.
     It is independent from any trigger_message-settings. It is simply always
@@ -124,10 +125,10 @@ class NewClock(bus_message.Message):
         """
         set the clock to clock. This is the new clock, the engine is using.
         """
-        bus_message.Message.__init__(self)
+        message.Message.__init__(self)
         self.clock = clock
 
-class NewBaseTime(bus_message.Message):
+class NewBaseTime(message.Message):
     """
     This message is triggered everytime, the engine got a new BaseTime. This
     message simply is send everytime a gstreamer change-state message is
@@ -139,10 +140,10 @@ class NewBaseTime(bus_message.Message):
     """
 
     def __init__(self, base_time):
-        bus_message.Message.__init__(self)
+        message.Message.__init__(self)
         self.base_time = base_time
 
-class PlayerError(bus_message.Message):
+class PlayerError(message.Message):
     """
     Sent over the message bus if the player encountered an issue.
 
@@ -150,7 +151,7 @@ class PlayerError(bus_message.Message):
     @type error: str
     """
     def __init__(self, error):
-        bus_message.Message.__init__(self)
+        message.Message.__init__(self)
         self.error = error
 
 # FIXME: need for a "codec missing" message
@@ -255,6 +256,7 @@ class Player(log.Loggable):
         self._player_engine_registry = registry
 
         self._uri = None
+        self._visualisation = None
 
         application = common.application
         self._audiosink = application.config.get_option('audiosink',
@@ -417,6 +419,8 @@ class Player(log.Loggable):
             return
 
         self._uri = uri
+        if uri == None:
+            return
         e_reg = self._player_engine_registry
         media_manager = common.application.media_manager
         bus = common.application.bus
@@ -464,6 +468,7 @@ class Player(log.Loggable):
                 else:
                     self._engine.audio_sink = self._new_audio_sink()
                 self._engine.uri = uri
+                self._engine.visualisation = self.visualisation
                 self._loaded = True
             except NoEngineFound:
                 self.warning("No Engine found for the uri %s" % uri)
@@ -515,6 +520,14 @@ class Player(log.Loggable):
     def subtitle_uri__get(self):
         return self._sub_uri
 
+    def visualisation__get(self):
+        return self._visualisation
+
+    def visualisation__set(self, new_visu):
+        # That feels bad to me...
+        self._visualisation = new_visu
+        
+
     def subtitle_callback__set(self, callback):
         self._sub_callback = callback
         self._update_subtitle_handoff()
@@ -551,6 +564,10 @@ class Player(log.Loggable):
         base_time = message.base_time
         if self._sub_uri != None:
             self._sub_pipeline.set_base_time(base_time)
+            self._sub_pipeline.seek(1, gst.FORMAT_TIME,
+                                gst.SEEK_FLAG_FLUSH | gst.SEEK_FLAG_KEY_UNIT,
+                                gst.SEEK_TYPE_SET, self.position,
+                                gst.SEEK_TYPE_SET, self.duration)
             self._sub_pipeline.set_state(gst.STATE_PLAYING)
             self.debug("Got new basetime, starting to play!")
 
@@ -627,9 +644,10 @@ class Player(log.Loggable):
     def _new_audio_sink(self): 
         # make a new sink
         audio_sink = gst.element_factory_make(self._audiosink)
-        for key, value in self._audiosettings:
+        for key, value in self._audiosettings.iteritems():
             audio_sink.set_property(key, value)
-
+        return audio_sink
+    
     def _make_subtitle_pipeline(self):
         uri_dec = "uridecodebin name=src ! "\
                   "text/plain;text/x-pango-markup ! " \
