@@ -1,11 +1,11 @@
 # -*- coding: utf-8 -*-
 # Elisa - Home multimedia server
-# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
+# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
 # All rights reserved.
 #
 # This file is available under one of two license agreements.
 #
-# This file is licensed under the GPL version 2.
+# This file is licensed under the GPL version 3.
 # See "LICENSE.GPL" in the root of this distribution including a special
 # exception to use Elisa with Fluendo's plugins.
 #
@@ -21,11 +21,12 @@ Plugins load/management support
 
 __maintainer__ = 'Philippe Normand <philippe@fluendo.com>'
 
+from elisa.core import __version__ as elisa_version
 from elisa.core import log, config
 from elisa.core.plugin import Plugin
 from elisa.core.component import ComponentError, UnMetDependency, InitializeFailure
 from elisa.core.component import UnSupportedPlatform
-from elisa.core.utils import classinit
+from elisa.core.utils import classinit, misc
 from elisa.core import common
 from elisa.extern import path
 from sets import Set
@@ -34,6 +35,9 @@ import os, sys, re, gc
 import inspect
 import types
 import new
+from twisted.internet import defer
+
+DEFAULT_PLUGIN_DIR = os.path.expanduser('~/.elisa/plugins')
 
 # FIXME: why is this shortcut not in common ?
 def get_component_class(component_path):
@@ -42,16 +46,8 @@ def get_component_class(component_path):
     @rtype: L{elisa.core.component.Component} class
     """
     plugin_registry = common.application.plugin_registry
-    # FIXME: that try/except is not useful anymore if get_component_class
-    #        returns None instead of raising ComponentNotFound
-    try:
-        component_class = plugin_registry.get_component_class(component_path)
-        return component_class
-    except:
-        log.warning("plugin_registry", "Component %r not found", component_path)
-        from elisa.core.component import Component as ComponentClass
-        return ComponentClass
-
+    component_class = plugin_registry.get_component_class(component_path)
+    return component_class
 
 class PluginNotFound(Exception):
 
@@ -71,8 +67,6 @@ class ComponentNotFound(Exception):
     def __str__(self):
         return "Component %r not found" % self.component_name
 
-
-
 class PluginRegistry(log.Loggable):
     """
     The PluginRegistry is responsible to find all the Plugins. Plugins
@@ -110,56 +104,62 @@ class PluginRegistry(log.Loggable):
         self._plugin_instances = {}
         self._app_config = application_config
         self._init_local_plugins_dir()
-
+        
     def _init_local_plugins_dir(self):
         """
         Register the local plugins directory in pkg_resources. The
         directory name is found in the application's config in
         'plugins_dir' option which is in the 'general' section
         """
-        default = '~/.elisa/plugins'
-
-        expanded_default = os.path.expanduser(default)
-        if not os.path.exists(expanded_default):
+        default = DEFAULT_PLUGIN_DIR
+        
+        if not os.path.exists(default):
             try:
-                os.makedirs(expanded_default)
+                os.makedirs(default)
             except OSError, e:
-                self.warning("Could not make directory %r: %s",expand_default,
-                             e)
+                self.warning("Could not make directory %r: %s", default, e)
                 default = ''
             else:
-                default_pkg = os.path.join(expanded_default, '__init__.py')
+                default_pkg = os.path.join(default, '__init__.py')
                 if not os.path.exists(default_pkg):
                     try:
-                        open(default_pkg,'w').close()
+                        open(default_pkg, 'w').close()
                     except IOError, error:
                         self.warning("Could not create %r: %r", default_pkg,
                                      error)
                         default = ''
-                    
-        plugin_path = os.getenv('ELISA_PLUGIN_PATH', default)
-        directories = plugin_path.split(':')
 
+        directories = misc.env_var_explode_list('ELISA_PLUGIN_PATH', default)
         for directory in directories:
             if not directory:
                 continue
-            if directory.startswith('~'):
-                directory = os.path.expanduser(directory)
-            else:
-                directory = os.path.abspath(directory)
+
+            directory = os.path.expanduser(directory)
 
             if directory.endswith(os.path.sep):
                 directory = directory[:-1]
 
             if not os.path.exists(os.path.join(directory, '__init__.py')):
                 self.warning("__init__.py file missing in %r; "
-                             "skipping plugin directory registration", directory)
+                             "skipping plugin directory registration",
+                             directory)
             else:
-                self.debug("Plugin directory: %r", directory)
+                self.info("Plugin directory: %r", directory)
                 self._plugin_directories.append(directory)
 
+        if default not in self._plugin_directories:
+            self.info("Adding default plugins directory: %r", default)
+            self._plugin_directories.insert(0, default)
+
+    def _init_environment(self, plugin_directories):
+        if not plugin_directories:
+            plugin_directories = self._plugin_directories
+
+        self.info("Initializing environment for plugin directories %s",
+                  plugin_directories)
+        
         # set up our custom environment where plugins are located
-        environment = pkg_resources.Environment(self._plugin_directories)
+        environment = pkg_resources.Environment(plugin_directories)
 
         working_set = pkg_resources.working_set
 
@@ -172,7 +172,7 @@ class PluginRegistry(log.Loggable):
         else:
             self.warning("Please upgrade setuptools if you want to "\
                          "be able to use additional plugins  "\
-                         "from %s" % self._plugin_directories)
+                         "from %r", plugin_directories)
 
     def plugins__get(self):
         return self._plugin_instances
@@ -187,20 +187,19 @@ class PluginRegistry(log.Loggable):
         @type plugin_class:  class
         """
         plugin_class.load_config()
-        plugin_class.load_translations(common.application.translator)
-        
+        if common.application and common.application.translator:
+            plugin_class.load_translations(common.application.translator)
+
         try:
             errors = plugin_class.initialize()
         except (UnMetDependency,InitializeFailure), error:
             self.warning(error)
         else:
-            for error in errors:
-                self.warning(error)
-
             name = plugin_class.name
             self._plugin_classes[name] = plugin_class
-            self.info("Loaded the plugin %r", name)
-                
+            self.info("Loaded the plugin %r version %r from %r", name,
+                      plugin_class.version, plugin_class.directory)
+
             names = plugin_class.components.keys()
             names.sort()
             component_names = ', '.join(names)
@@ -248,76 +247,132 @@ class PluginRegistry(log.Loggable):
 
         return plugin
 
-    def get_plugin_path(self, plugin_name):
-        """
-        Retrieve the absolute filesystem path of the Python module
-        where the given plugin is defined.
-
-        @param plugin_name: name of the Plugin to look for
-        @type plugin_name:  string
-        @rtype:             string
-        """
-        path = ""
-        plugin_class = self._plugin_classes.get(plugin_name)
-        if plugin_class:
-            if not plugin_class.directory:
-                plugin_dir = inspect.getsourcefile(plugin_class)
-                plugin_class.directory = os.path.dirname(plugin_dir)
-            path = plugin_class.directory
-        return path
-
-    def load_plugins(self):
+    def load_plugins(self, plugin_directories=None):
         """
         Find and register all Plugin classes found by setuptools and
         in the plugins directory.
+
+        @keyword plugin_directories: list of directories to scan for
+                                     plugins default:
+                                     [DEFAULT_PLUGIN_DIR,] + $ELISA_PLUGIN_PATH
+        @type plugin_directories:    list of strings
         """
+        self._init_environment(plugin_directories=plugin_directories)
+        
+        self._register_core()
+        
         # load plugins registered via various means
         self._load_eggs()
-        self._load_packages()
-        self._load_modules()
+##         self._load_packages()
+##         self._load_modules()
         self._load_by_config()
 
-        # check plugin dependencies
-        self._check_plugin_dependencies()
+        self._check_plugin_dependencies() 
+
+    def _register_core(self):
+        plugin_class = new.classobj('Core', (Plugin,),{})
+        plugin_class.name = 'core'
+        plugin_class.version = elisa_version
+        plugin_class.plugin_dependencies = []
+
+        import elisa.core
+        directory = os.path.dirname(elisa.core.__file__)
+        plugin_class.directory = directory
+
+        elisa_root = os.path.dirname(os.path.dirname(directory))
+
+        dist = None
+        distribution_file = ""
+        try:
+            dist = pkg_resources.require('elisa')[0]
+        except pkg_resources.DistributionNotFound:
+            if os.path.isdir(elisa_root):
+                dists = [ distribution for distribution in
+                          pkg_resources.find_distributions(elisa_root, True)
+                          if distribution.project_name == 'elisa' and
+                          distribution.version == elisa_version ]
+                if dists:
+                    dist = dists[0]
+
+        if dist and os.path.isfile(dist.location):
+            distribution_file = dist.location
+
+        plugin_class.distribution_file = distribution_file
+        
+        self.register_plugin(plugin_class)
 
     def _check_plugin_dependencies(self):
-        all_plugins = self._plugin_classes.keys()
-
-
         plugin_classes = self._plugin_classes.copy()
         for plugin_name, plugin_class in plugin_classes.iteritems():
-            for dependency in plugin_class.plugin_dependencies:
-                if dependency not in all_plugins:
-                    msg = "%r plugin is required by %s plugin" % (dependency,
-                                                                  plugin_name)
-                    self.warning(msg)
-                    #del self._plugin_classes[plugin_name]
+            try:
+                self.check_interplugin_dependencies(plugin_class)
+            except UnMetDependency, exception:
+                msg = "plugin %r is missing the following" \
+                      " dependencies: %s" \
+                       % (plugin_name, exception.error_message)
+                self.warning(msg)
+                continue
 
-
-        self._check_component_dependencies()
-        self._check_component_dependencies()
-
-    def _check_component_dependencies(self):
-        plugin_classes = self._plugin_classes.copy()
-
-        for plugin_name, plugin_class in plugin_classes.iteritems():
             # for each component, check eventual inter-component deps
             components = plugin_class.components.copy()
-            for component, informations in plugin_class.components.iteritems():
-                dependencies = informations.get('component_dependencies',[])
-                for dependency in dependencies:
-                    plugin, cmpt = dependency.split(':')
-                    other_plugin = self._plugin_classes.get(plugin)
-                    if not other_plugin or cmpt not in other_plugin.components:
-                        self.warning("%r component is required by %s component",
-                                     dependency, component)
-                        self.warning("Removing component %r from plugin %r",
-                                     component, plugin_name)
-                        del components[component]
-                        break
+            for component_name, informations in components.iteritems():
+                try:
+                    self.check_intercomponent_dependencies(plugin_class,
+                                                           component_name)
+                except UnMetDependency, exception:
+                    msg = "component %r is missing the following" \
+                          " dependencies: %s" \
+                          % (component_name, exception.error_message)
+                    self.warning(msg)
+                    continue
 
-            if components:
-                plugin_class.components = components
+    def check_interplugin_dependencies(self, plugin_class):
+        """ 
+        Check that all the plugins that L{plugin_class} depends on are loaded.
+
+        @param plugin_class: Plugin's class
+        @type plugin_class:  class
+        
+        @raises: L{elisa.core.component.UnMetDependency} if at least one
+                 dependency is not satisfied; the list of unmet dependencies
+                 is passed.
+        """
+        unmet_dependencies = []
+
+        for dependency in plugin_class.plugin_dependencies:
+            if dependency not in self._plugin_classes.keys():
+                unmet_dependencies.append(dependency)
+
+        if len(unmet_dependencies) > 0:
+            raise UnMetDependency(plugin_class, unmet_dependencies)
+        
+    def check_intercomponent_dependencies(self, plugin, component_name):
+        """ 
+        Check that all the components that L{component} depends on are
+        available.
+
+        @param plugin:         plugin containing the tested component
+        @type plugin:          Plugin
+        @param component_name: component which dependencies are to be checked
+        @type component_name:  string
+        
+        @raises: L{elisa.core.component.UnMetDependency} if at least one
+                 dependency is not satisfied; the list of unmet dependencies
+                 is passed.
+        """
+        informations = plugin.components[component_name]
+        dependencies = informations.get('component_dependencies',[])
+        unmet_dependencies = []
+
+        for dependency in dependencies:
+            dependency_plugin, dependency_component = dependency.split(':')
+            dependency_plugin_class = self._plugin_classes.get(dependency_plugin)
+            if not dependency_plugin_class or \
+               dependency_component not in dependency_plugin_class.components:
+                unmet_dependencies.append(dependency)
+
+        if len(unmet_dependencies) > 0:
+            raise UnMetDependency(component_name, unmet_dependencies)
 
     def _load_eggs(self):
         entrypoints = list(pkg_resources.iter_entry_points('elisa.plugins'))
@@ -327,19 +382,33 @@ class PluginRegistry(log.Loggable):
         for entrypoint in entrypoints:
             try:
                 py_object = entrypoint.load()
-            except:
-                self.warning("Error loading plugin %r" % entrypoint.module_name)
-                common.application.handle_traceback()
+            except Exception, exc:
+                if common.application:
+                    trace_path = common.application.log_traceback()
+                    self.warning("Error loading plugin %r. A full traceback can"\
+                                 " be found here: %s" % (entrypoint.module_name,
+                                                         trace_path))
+                else:
+                    self.warning("Error loading plugin: " + str(exc))
                 continue
-
+            
             full_path = inspect.getsourcefile(py_object)
             plugin_dir = os.path.dirname(full_path)
 
+            distribution_file = ""
+            if os.path.isfile(entrypoint.dist.location):
+                distribution_file = entrypoint.dist.location
+            
             if type(py_object) == types.ModuleType:
+                module_name = entrypoint.module_name
                 dirname = os.path.basename(os.path.split(full_path)[0])
                 plugin_class = new.classobj(dirname.title(), (Plugin,),{})
+                plugin_class.__module__ = module_name
+                
                 # TODO: should that be configurable?
-                config_path = os.path.join(plugin_dir, "plugin.conf")
+                plugin_conf = 'plugin.conf'
+                
+                config_path = plugin_class.get_resource_file(plugin_conf)
                 if not os.path.exists(config_path):
                     self.warning("Plugin config file not found for %r plugin",
                                  entrypoint.name)
@@ -349,7 +418,8 @@ class PluginRegistry(log.Loggable):
                 plugin_class = py_object
 
             plugin_class.directory = plugin_dir
-
+            plugin_class.distribution_file = distribution_file
+            
             assert issubclass(plugin_class, Plugin), \
                    '%r is not a valid Plugin!' % plugin_class
 
@@ -380,16 +450,18 @@ class PluginRegistry(log.Loggable):
                     try:
                         module = __import__(module_name, {}, {},
                                             [module_filename,])
-                    except ImportError, error:
-                        self.warning("Could not import module %s" % filename)
-                        common.application.handle_traceback()
-                        continue
+                    except Exception, error:
+                        trace_path = common.application.log_traceback()
+                        self.warning("Could not import module %s. The" \
+                                     " traceback can be found at %s" % (
+                                        filename, trace_path))
                     else:
                         modules.append(module)
 
                     sys.path.remove(parent)
             except OSError, error:
                 self.warning(error)
+                continue
                 
         for module in modules:
             for symbol_name in dir(module):
@@ -444,21 +516,24 @@ class PluginRegistry(log.Loggable):
         @rtype:                L{elisa.core.component.Component}
         @raise InitializeFailure: When the component failed to initialize
         @raise ComponentNotFound: When the component could not be found in any plugin
+        @raise UnMetDependency:   When the component misses a dependency
         """
         component = None
 
         self.debug("Trying to create %r component" % component_path)
 
-        try:
-            infos = self._get_component_infos(component_path)
-        except PluginNotFound, error:
-            self.warning(error)
-            raise ComponentNotFound(component_path)
+        infos = self._get_component_infos(component_path)
 
         ComponentClass, plugin, component_path_tuple = infos
 
         if ComponentClass:
             ComponentClass.name = component_path_tuple[1]
+
+            # bind the Component to its plugin early as it's needed by
+            # check_dependencies()
+            if not ComponentClass.plugin:
+                ComponentClass.plugin = plugin
+
             instance_id = component_path_tuple[2]
             self.log("Instantiating Component %r from %r",
                      ComponentClass.name, component_path_tuple[0])
@@ -467,9 +542,23 @@ class PluginRegistry(log.Loggable):
             # component.name = component_name
             component.path = component_path_tuple[3]
             component.id = instance_id
-            component.plugin = plugin
             component.load_config(self._app_config)
-            component.initialize()
+            res = component.initialize()
+            if isinstance(res, defer.Deferred):
+                # NOTE: do this here instead of using maybeDeferred because
+                # callers don't expect a deferred. A lot of tests are calling
+                # create_component so this is the least invasive way of doing
+                # this.
+                def component_initialize_done(result):
+                    self.info("Component %s loaded" % component.name)
+
+                    return component
+
+                self.info("Component %s still loading" % component.name)
+                res.addCallback(component_initialize_done)
+
+                return res
+
             self.info("Component %s loaded" % component.name)
         else:
             raise ComponentNotFound(component_path)
@@ -498,6 +587,10 @@ class PluginRegistry(log.Loggable):
             plugin = self.get_plugin_with_name(plugin_name)
             self.debug("Found plugin %r, searching for Component %r",
                        plugin_name, component_name)
+
+            # lazily check plugin's python deps
+            plugin.check_dependencies()
+
             ComponentClass = self._import_component(plugin, component_name)
 
         return (ComponentClass, plugin, component_path_tuple)
@@ -512,10 +605,15 @@ class PluginRegistry(log.Loggable):
 
             if isinstance(path, basestring):
                 # relative.path:ComponentClass
+
+                # lazily check the dependencies of the Component before trying
+                # to import it.
+                plugin.check_component_dependencies(component_name)
+                
                 mod_class = path.split(':')
                 plugin_path = plugin.directory
                 old_path = sys.path[:]
-                sys.path.insert(0, os.path.dirname(plugin_path))
+                sys.path.insert(0, os.path.abspath(os.path.dirname(plugin_path)))
                 package_name = os.path.basename(plugin_path)
                 if package_name:
                     full_path = "%s.%s" % (package_name,mod_class[0])
@@ -526,16 +624,18 @@ class PluginRegistry(log.Loggable):
                                         {}, {}, [mod_class[1]])
                     Class = getattr(module, mod_class[1])
                 except (ImportError, AttributeError), error:
-                    self.warning("Could not import component %s.%s",
-                                 plugin.name, path)
+                    trace_path = common.application.log_traceback()
+                    self.warning("Could not import component %s.%s. A full"\
+                                 " traceback can be found at %s" %
+                                 (plugin.name, path, trace_path))
                     sys.path = old_path
-                    common.application.handle_traceback()
                 else:
                     self.debug("Imported %r from %s", Class, plugin_path)
                 sys.path = old_path
-                    
+
             else:
                 Class = path
+
         return Class
 
     def get_component_class(self, component_path):
@@ -545,6 +645,7 @@ class PluginRegistry(log.Loggable):
         @type component_path:  string
         @rtype: L{elisa.core.component.Component} class
         """
+        self.debug("Retrieving component class of %r", component_path)
         infos = self._get_component_infos(component_path)
         ComponentClass, plugin, path_tuple = infos
 
@@ -596,25 +697,3 @@ class PluginRegistry(log.Loggable):
                          component_path)
 
         return component_path_tuple
-
-
-    def create_components(self, component_path_list):
-        """ Use the PluginRegistry to create a list of Components.
-        See create_component for more details
-
-        @param component_path: list of information to locate the Component
-                               to load
-        @type component_path:  list
-        @rtype:                list of L{elisa.core.component.Component}
-        """
-        components = []
-
-        for path in component_path_list:
-            try:
-                component = self.create_component(path)
-                components.append(component)
-            except (PluginNotFound, ComponentNotFound,
-                    InitializeFailure), error:
-                self.warning(error)
-
-        return components
