@@ -1,10 +1,10 @@
 # Elisa - Home multimedia server
-# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
+# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
 # All rights reserved.
 #
 # This file is available under one of two license agreements.
 #
-# This file is licensed under the GPL version 2.
+# This file is licensed under the GPL version 3.
 # See "LICENSE.GPL" in the root of this distribution including a special
 # exception to use Elisa with Fluendo's plugins.
 #
@@ -18,20 +18,42 @@ from elisa.core.tests.elisa_test_case import ElisaTestCase
 from elisa.core import plugin_registry
 from elisa.core import config
 from elisa.core import plugin
-from elisa.base_components import activity
-import inspect, os
+from elisa.core.component import Component, UnMetDependency
+from elisa.base_components.activity import Activity
+import inspect, os, platform
 import tempfile, shutil
+from twisted.internet import defer
 
-class SomeActivity(activity.Activity):
+class SomeActivity(Activity):
     name = 'some_activity'
 
+class DeferredInitializeActivity(Activity):
+    name = 'deferred_initialize_activity'
+
+    def initialize(self):
+        return defer.succeed(None)
+
 class SomePlugin(plugin.Plugin):
     name = 'test'
-    components = {'some_activity':{'path':SomeActivity}}
+    components = {
+            'some_activity': {'path': SomeActivity},
+            'deferred_initialize_activity': {'path': DeferredInitializeActivity}
+        }
 
 class FooPlugin(plugin.Plugin):
     name = 'foo'
 
+class MissingDepsPlugin(plugin.Plugin):
+    name = 'missing_deps'
+    plugin_dependencies = ['unexistent_plugin']
+    components = {'missing_deps_component':
+                   {'path': 'fake:MissingDepsComponent',
+                    'component_dependencies': ['unexistent_plugin:component']},
+                  'missing_external_deps_component':
+                   {'path': 'fake:MissingDepsComponent',
+                    'external_dependencies': ['fjezivfihahfue']}
+                 }
+
 SOME_PLUGIN="""\
 from elisa.core import plugin
 from elisa.base_components import activity
@@ -80,6 +102,10 @@ class TestPluginRegistry(ElisaTestCase):
     def test_plugin_loading(self):
         """ Check the plugins declared in the config are loaded
         """
+        
+        if platform.system() == 'Windows':
+            raise unittest.SkipTest("Does not works under windows, need investigation")
+         
         conf = config.Config('empty.conf')
         os.environ['ELISA_PLUGIN_PATH'] = self._plugins_dir
         section = {'plugins':['test', 'foo']}
@@ -113,6 +139,8 @@ class TestPluginRegistry(ElisaTestCase):
         self.assertEquals(test_plugin.__name__, 'SomePlugin')
         self.assertEquals(foo_plugin.__name__, 'FooPlugin')
 
+    test_plugin_loading.skip = "The plugin_registry can no longer load plugins from python modules"
+
     def test_plugin_instantiation(self):
         """ Check Plugin instances are lazily referenced by the manager
         """
@@ -126,6 +154,24 @@ class TestPluginRegistry(ElisaTestCase):
         self.failUnless(isinstance(plugin1, SomePlugin))
         self.failUnless(plugin1 == plugin2)
 
+    def test_plugin_instantiation_failures(self):
+        """ Test several error cases of plugin instantiation.
+        """
+        conf = config.Config('empty.conf')
+        manager = plugin_registry.PluginRegistry(conf)
+
+        manager.register_plugin(MissingDepsPlugin)
+        plugin = manager.get_plugin_with_name('missing_deps')
+
+        self.failUnless(isinstance(plugin, MissingDepsPlugin))
+
+        self.failUnlessRaises(UnMetDependency,
+                              manager.check_interplugin_dependencies, plugin)
+
+        self.failUnlessRaises(UnMetDependency,
+                              manager.check_intercomponent_dependencies,
+                              plugin, 'missing_deps_component')
+
     def test_component_creation(self):
         """ Check Components are correctly instantiated by the manager
         """
@@ -135,10 +181,37 @@ class TestPluginRegistry(ElisaTestCase):
         manager = plugin_registry.PluginRegistry(conf)
         manager.register_plugin(SomePlugin)
 
-        activities = manager.create_components(['test:some_activity',])
-        self.assertEqual(len(activities), 1)
-        self.failUnless(isinstance(activities[0], activity.Activity))
-        self.failUnless(isinstance(activities[0], SomeActivity))
+        activity = manager.create_component('test:some_activity')
+        self.failUnless(isinstance(activity, Activity))
+        self.failUnless(isinstance(activity, SomeActivity))
+
+    def test_component_creation_deferred_initialize(self):
+        """Check components that return a deferred from Component.initialize()
+        """
+        conf = config.Config('empty.conf')
+        conf.set_option('activities', ['test:deferred_initialize_activity',],
+                        'plugin_registry')
+        manager = plugin_registry.PluginRegistry(conf)
+        manager.register_plugin(SomePlugin)
+
+        def create_component_done(activity):
+            self.failUnless(isinstance(activity, Activity))
+            self.failUnless(isinstance(activity, DeferredInitializeActivity))
+
+        dfr = manager.create_component('test:deferred_initialize_activity')
+        dfr.addCallback(create_component_done)
+
+        return dfr
+
+    def test_component_creation_failures(self):
+        conf = config.Config('empty.conf')
+        manager = plugin_registry.PluginRegistry(conf)
+
+        manager.register_plugin(MissingDepsPlugin)
+
+        self.failUnlessRaises(UnMetDependency,
+                              manager.create_component,
+                             'missing_deps:missing_external_deps_component')
 
     def test_multiple_component_creation(self):
         """ Check the manager can create multiple instance of the same component
@@ -151,14 +224,17 @@ class TestPluginRegistry(ElisaTestCase):
         manager = plugin_registry.PluginRegistry(conf)
         manager.register_plugin(SomePlugin)
 
-        activities = manager.create_components(conf_activities)
-        self.assertEqual(len(activities), 2)
+        activities = []
         index = 1
-        for component in activities:
+        for activity_path in conf_activities:
+            component = manager.create_component(activity_path)
             self.assertEqual(component.id, index)
-            self.failUnless(isinstance(component, activity.Activity))
+            self.failUnless(isinstance(component, Activity))
             self.failUnless(isinstance(component, SomeActivity))
             index += 1
+            activities.append(component)
+
+        self.assertEqual(len(activities), 2)
 
     def test_create_component(self):
         """
@@ -173,7 +249,7 @@ class TestPluginRegistry(ElisaTestCase):
 
         act = manager.create_component('test:some_activity')
         self.failUnless(act)
-        self.failUnless(isinstance(act, activity.Activity))
+        self.failUnless(isinstance(act, Activity))
         self.failUnless(isinstance(act, SomeActivity))
 
     def test_get_component_class(self):
@@ -198,7 +274,7 @@ class TestPluginRegistry(ElisaTestCase):
 
         c = plugin_registry.get_component_class('test:some_activity')
         self.failUnless(c)
-        self.failUnless(issubclass(c, activity.Activity))
+        self.failUnless(issubclass(c, Activity))
         self.assertEqual(c, SomeActivity)
 
 
@@ -216,14 +292,3 @@ class TestPluginRegistry(ElisaTestCase):
         self.assertEqual(('A', 'B', 0, 'A:B'), manager._split_component_path('A:B'))
         self.assertEqual(('A', 'B', 1, 'A:B:1'), manager._split_component_path('A:B:1'))
         self.assertEqual((), manager._split_component_path('A:B:c'))
-
-    def test_get_plugin_path(self):
-        """
-        """
-        conf = config.Config('empty.conf')
-        manager = plugin_registry.PluginRegistry(conf)
-        manager.register_plugin(SomePlugin)
-
-        path = manager.get_plugin_path('test')
-        abspath = os.path.dirname(inspect.getsourcefile(self.__class__))
-        self.assertEqual(abspath, path)
