@@ -1,11 +1,11 @@
 # -*- coding: utf-8 -*-
 # Elisa - Home multimedia server
-# Copyright (C) 2006,2007 Fluendo Embedded S.L. (www.fluendo.com).
+# Copyright (C) 2006-2008 Fluendo Embedded S.L. (www.fluendo.com).
 # All rights reserved.
 #
 # This file is available under one of two license agreements.
 #
-# This file is licensed under the GPL version 2.
+# This file is licensed under the GPL version 3.
 # See "LICENSE.GPL" in the root of this distribution including a special
 # exception to use Elisa with Fluendo's plugins.
 #
@@ -14,6 +14,10 @@
 # See "LICENSE.Elisa" in the root directory of this distribution package
 # for details on that license.
 
+"""
+Generell profiling methods
+"""
+
 
 __maintainer__ = 'Florian Boucault <florian@fluendo.com>'
 
@@ -29,6 +33,8 @@ def profileit(printlines=1):
             prof.close()
             stats = hotshot.stats.load("profiling.data")
             stats.strip_dirs()
+            #stats.sort_stats('cumul', 'calls')
+            #stats.sort_stats('calls', 'cumul')
             stats.sort_stats('time', 'calls')
             print ">>>---- Begin profiling print"
             stats.print_stats(printlines)
